from django.urls import path
from hydrothings import SensorThingsAPI


hydrothings_api_1_1 = SensorThingsAPI(
    version='1.1',
    backend='frostserver'
)

hydrothings_api_1_0 = SensorThingsAPI(
    version='1.0',
    backend='frostserver'
)

urlpatterns = [
    path('v1.0/', hydrothings_api_1_0.urls),
    path('v1.1/', hydrothings_api_1_1.urls)
]
